'use strict';

var LOG_LEVELS;
(function (LOG_LEVELS) {
    LOG_LEVELS["DEBUG"] = "log";
    LOG_LEVELS["INFO"] = "info";
    LOG_LEVELS["WARN"] = "warn";
    LOG_LEVELS["ERROR"] = "error";
})(LOG_LEVELS || (LOG_LEVELS = {}));

// Determine whether number is inbetween a set of numbers
const isBetween = (value, a, b) => value >= Math.min(a, b) && value <= Math.max(a, b);

const breakpointHandler = (sizes, breakpoints) => {
    let currentConfines = {};
    if (Array.isArray(sizes) || !breakpoints) {
        // Ad is not using responsive plugin
        return { sizesSpecified: !!sizes.length };
    }
    Object.keys(breakpoints).forEach((key) => {
        const { from, to } = breakpoints[key];
        if (isBetween(window.innerWidth, from, to)) {
            currentConfines = {
                from,
                to,
                breakpoint: key,
                sizesSpecified: !!(sizes[key] && sizes[key].length > 0),
            };
        }
    });
    return currentConfines;
};

var dispatchEvent = (id, level, creator, message, eventData) => {
    const event = new CustomEvent(`adjs:${id}:event`, {
        detail: {
            level,
            creator,
            message,
            data: eventData,
        },
    });
    window.dispatchEvent(event);
};

const loadScript = (url, attributes = {}) => new Promise((resolve) => {
    const scriptTag = document.createElement('script');
    scriptTag.src = url;
    scriptTag.onload = () => resolve();
    Object.keys(attributes).forEach((key) => {
        const value = attributes[key];
        if (typeof value === 'boolean') {
            if (!value) {
                return;
            }
            scriptTag.setAttribute(key, key);
            return;
        }
        scriptTag.setAttribute(key, value);
    });
    document.getElementsByTagName('head')[0].appendChild(scriptTag);
});

class DfpAd {
    constructor(ad) {
        this.ad = ad;
        DoubleClickForPublishers.prepare();
        const { el: { id }, configuration: { sizes, targeting, path, breakpoints, collapsible }, } = ad;
        if (!id) {
            throw new Error('Ad does not have an id');
        }
        if (!sizes) {
            throw new Error('Sizes must be defined.');
        }
        if (!path) {
            throw new Error('Ad Path must be defined.');
        }
        this.id = id;
        this.sizes = sizes;
        this.breakpoints = breakpoints;
        googletag.cmd.push(() => {
            this.slot = googletag.defineSlot(path, Array.isArray(sizes) ? sizes : [0, 0], this.id);
            if (collapsible) {
                this.slot.setCollapseEmptyDiv(true, true);
            }
            if (targeting) {
                Object.entries(targeting)
                    .forEach(([key, value]) => {
                    this.slot.setTargeting(key, value);
                });
                dispatchEvent(Number(this.id.substring(this.id.length, this.id.length - 1)), LOG_LEVELS.INFO, 'DFP Network', 'Targeting detected for ad. Adding to configuration.');
            }
            if (!Array.isArray(sizes) && breakpoints) {
                const sizing = googletag.sizeMapping();
                Object.entries(breakpoints).map(([device, dimensions]) => {
                    sizing.addSize([dimensions.from, 1], sizes[device] || []);
                });
                this.slot.defineSizeMapping(sizing.build());
            }
            this.slot.addService(googletag.pubads());
            googletag.display(this.id);
        });
    }
    render() {
        return this.refresh();
    }
    clear() {
        return new Promise((resolve) => {
            googletag.cmd.push(() => {
                const { slot } = this;
                googletag.pubads().clear([slot]);
                this.ad.isEmpty = true;
                resolve();
            });
        });
    }
    // TODO: Clean up event listeners
    refresh(resetCorrelator = false) {
        return new Promise((resolve) => {
            if (!breakpointHandler(this.sizes, this.breakpoints).sizesSpecified) {
                this.clear().then(resolve);
                return;
            }
            googletag.cmd.push(() => {
                const { slot } = this;
                googletag.pubads().addEventListener('slotRenderEnded', (event) => {
                    if (event.slot === slot) {
                        this.ad.isEmpty = event.isEmpty;
                        dispatchEvent(Number(this.id.substring(this.id.length, this.id.length - 1)), LOG_LEVELS.INFO, 'DFP Network', 'Ad slot has been rendered.');
                        resolve();
                    }
                });
                googletag.pubads().refresh([slot], { changeCorrelator: resetCorrelator });
                if (!slot.getContentUrl()) {
                    this.ad.isEmpty = true;
                    dispatchEvent(Number(this.id.substring(this.id.length, this.id.length - 1)), LOG_LEVELS.WARN, 'DFP Network', 'Ad sizes missing. Bypassing ad.');
                    resolve();
                }
            });
        });
    }
    // Cannot undo this action
    destroy() {
        return new Promise((resolve) => {
            googletag.cmd.push(() => {
                const { slot } = this;
                googletag.destroySlots([slot]);
                this.ad.isEmpty = true;
                resolve();
            });
        });
    }
}
function loadGPT() {
    if (window.googletag && window.googletag.apiReady) {
        return;
    }
    loadScript('https://securepubads.g.doubleclick.net/tag/js/gpt.js', { async: true });
    window.googletag = window.googletag || {};
    window.googletag.cmd = window.googletag.cmd || [];
    window.googletag.cmd.push(() => {
        googletag.pubads().disableInitialLoad();
        googletag.pubads().enableAsyncRendering();
        googletag.enableServices();
    });
}
const DoubleClickForPublishers = {
    name: 'DoubleClick for Publishers',
    loaded: false,
    createAd(ad) {
        return new DfpAd(ad);
    },
    async prepare() {
        if (this.loaded) {
            return;
        }
        loadGPT();
        this.loaded = true;
    },
    async resetCorrelator() {
        googletag.pubads().updateCorrelator();
    },
};

module.exports = DoubleClickForPublishers;
