'use strict';

var LOG_LEVELS;
(function (LOG_LEVELS) {
    LOG_LEVELS["DEBUG"] = "log";
    LOG_LEVELS["INFO"] = "info";
    LOG_LEVELS["WARN"] = "warn";
    LOG_LEVELS["ERROR"] = "error";
})(LOG_LEVELS || (LOG_LEVELS = {}));

var dispatchEvent = (id, level, creator, message, eventData) => {
    const event = new CustomEvent(`adjs:${id}:event`, {
        detail: {
            level,
            creator,
            message,
            data: eventData,
        },
    });
    window.dispatchEvent(event);
};

/**
 * @param {function} cb Callback to be executed post timed throttle
 * @param {number} customDelay Time in Milliseconds
 */
let previousCall = 0;
var throttle = (cb, customDelay) => {
    const delay = customDelay || 200;
    const currentTime = new Date().getTime();
    if (currentTime - previousCall <= delay) {
        return;
    }
    previousCall = currentTime;
    return cb();
};

var _a, _b;
const MONITORING = Symbol('MONITORING');
const ON_SCROLL = Symbol('ON_SCROLL');
class ScrollMonitor {
    static monitorScroll() {
        if (ScrollMonitor[MONITORING]) {
            return;
        }
        window.addEventListener('scroll', ScrollMonitor[ON_SCROLL], false);
        ScrollMonitor[MONITORING] = true;
    }
}
_a = MONITORING, _b = ON_SCROLL;
ScrollMonitor.throttleDuration = 100;
ScrollMonitor.registeredAds = {};
ScrollMonitor.adCount = 0;
ScrollMonitor.subscribe = (id, element, offset = 0, onEnterViewport, onFullyEnterViewport, onExitViewport, enableByScroll) => {
    ScrollMonitor.monitorScroll();
    const existingAd = ScrollMonitor.getAdIfExists(id);
    if (existingAd) {
        if (onEnterViewport) {
            existingAd.onEnterViewport.push(onEnterViewport);
        }
        if (onFullyEnterViewport) {
            existingAd.onFullyEnterViewport.push(onFullyEnterViewport);
        }
        if (onExitViewport) {
            existingAd.onExitViewport.push(onExitViewport);
        }
        return;
    }
    const ad = {
        element,
        offset,
        inView: false,
        fullyInView: false,
        enableByScroll,
        hasViewBeenScrolled: false,
        onEnterViewport: onEnterViewport ? [onEnterViewport] : [],
        onFullyEnterViewport: onFullyEnterViewport ? [onFullyEnterViewport] : [],
        onExitViewport: onExitViewport ? [onExitViewport] : [],
    };
    ScrollMonitor.registeredAds[id] = ad;
    ScrollMonitor.evaulateCurrentViewability(ScrollMonitor.registeredAds[id], window.innerHeight);
    ++ScrollMonitor.adCount;
};
ScrollMonitor.unsubscribe = (id) => {
    if (!ScrollMonitor.registeredAds[id]) {
        return;
    }
    delete ScrollMonitor.registeredAds[id];
    --ScrollMonitor.adCount;
};
ScrollMonitor[_a] = false;
ScrollMonitor[_b] = () => throttle(() => {
    if (!ScrollMonitor.adCount) {
        return;
    }
    const windowHeight = window.innerHeight;
    Object.entries(ScrollMonitor.registeredAds).forEach(([key, ad]) => {
        ad.hasViewBeenScrolled = true;
        ScrollMonitor.evaulateCurrentViewability(ad, windowHeight);
    });
}, ScrollMonitor.throttleDuration);
ScrollMonitor.evaulateCurrentViewability = (ad, windowHeight) => {
    if (ad.enableByScroll && !ad.hasViewBeenScrolled) {
        return;
    }
    const bounding = ad.element.getBoundingClientRect();
    const inView = (bounding.top - ad.offset) <= windowHeight && (bounding.top + bounding.height) >= 0;
    const fullyInView = bounding.top >= 0 && bounding.bottom <= windowHeight;
    if (fullyInView && !ad.fullyInView) {
        ad.onFullyEnterViewport.forEach((fn) => fn());
    }
    if (inView && !ad.inView) {
        ad.onEnterViewport.forEach((fn) => fn());
    }
    if (!inView && ad.inView) {
        ad.onExitViewport.forEach((fn) => fn());
    }
    ad.inView = inView;
    ad.fullyInView = fullyInView;
};
ScrollMonitor.getAdIfExists = (id) => {
    return ScrollMonitor.registeredAds[id];
};

class GenericPlugin {
    constructor(ad) {
        if (!ad) {
            throw new Error(`An ad must be passed into the GenericPlugin class. If your Plugin inherits from GenericPlugin
		and overrides the constructor make sure you are calling "super" and that you are passing in an
		instance of an ad as the first parameter. Alternatively, you can hook into the onCreate method
		which gets called by the constructor.

		Example:
		js
		  class ExamplePlugin extends GenericPlugin {
			onCreate() {
			  console.log('Example Plugin Started Succesfully');
			}
		  }

		  // Or

		  class ExamplePlugin extends GenericPlugin {
			constructor(ad) {
			  super(ad);

			  console.log('Example Plugin Started Succesfully');
			}
		  }`);
        }
        this.ad = ad;
    }
    get name() {
        return this.constructor.name;
    }
    isEnabled(prop) {
        const value = this.ad.configuration[prop];
        if (typeof value === 'boolean') {
            return value;
        }
        return true;
    }
}

const ONE_SECOND = 1000;
class AutoRefresh extends GenericPlugin {
    constructor() {
        super(...arguments);
        this.timeInView = 0;
        this.isRefreshing = false;
        // Number of times the ad refreshed.
        this.refreshCount = 0;
        this.markAsInView = () => {
            if (this.timerReference) {
                return;
            }
            const { refreshRateInSeconds = 30, refreshLimit = null, refreshResetsCorrelator = false } = this.ad.configuration;
            dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'AutoRefresh Plugin', `Ad is in view. Beginning timer for ${refreshRateInSeconds}.`);
            this.timerReference = window.setInterval(async () => {
                this.timeInView += 1;
                // Determine whether the ad should refresh
                const underRefreshLimit = (refreshLimit === null || this.refreshCount < refreshLimit);
                if (!this.isRefreshing && this.timeInView >= refreshRateInSeconds && underRefreshLimit) {
                    this.isRefreshing = true;
                    this.refreshCount += 1;
                    // Once we hit the limit, clear the interval.
                    if (this.refreshCount === refreshLimit) {
                        clearInterval(this.timerReference);
                    }
                    dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'AutoRefresh Plugin', `Ad viewability met. Refreshing Ad and resetting timer.`);
                    await this.ad.refresh(refreshResetsCorrelator);
                    this.timeInView = 0;
                    this.isRefreshing = false;
                }
            }, ONE_SECOND);
        };
        this.markAsOutOfView = () => {
            if (!this.timerReference) {
                return;
            }
            dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'AutoRefresh Plugin', 'Ad is no longer in view.');
            clearInterval(this.timerReference);
            this.timerReference = undefined;
        };
    }
    afterRender() {
        dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'AutoRefresh Plugin', 'Adding monitoring for ad.');
        this.startMonitoringViewability();
        dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'AutoRefresh Plugin', 'Ad viewability is being monitored.');
    }
    beforeClear() {
        ScrollMonitor.unsubscribe(this.ad.el.id);
        dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'AutoRefresh Plugin', 'Ad viewability monitor has been removed.');
    }
    beforeDestroy() {
        ScrollMonitor.unsubscribe(this.ad.el.id);
        dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'AutoRefresh Plugin', 'Ad viewability monitor has been removed.');
    }
    startMonitoringViewability() {
        const { container, configuration: { offset = 0 }, el } = this.ad;
        ScrollMonitor.subscribe(el.id, container, offset, undefined, this.markAsInView, this.markAsOutOfView);
        this.timeInView = 0;
    }
}

module.exports = AutoRefresh;
