'use strict';

var LOG_LEVELS;
(function (LOG_LEVELS) {
    LOG_LEVELS["DEBUG"] = "log";
    LOG_LEVELS["INFO"] = "info";
    LOG_LEVELS["WARN"] = "warn";
    LOG_LEVELS["ERROR"] = "error";
})(LOG_LEVELS || (LOG_LEVELS = {}));

var dispatchEvent = (id, level, creator, message, eventData) => {
    const event = new CustomEvent(`adjs:${id}:event`, {
        detail: {
            level,
            creator,
            message,
            data: eventData,
        },
    });
    window.dispatchEvent(event);
};

/**
 * @param {function} cb Callback to be executed post timed throttle
 * @param {number} customDelay Time in Milliseconds
 */
let previousCall = 0;
var throttle = (cb, customDelay) => {
    const delay = customDelay || 200;
    const currentTime = new Date().getTime();
    if (currentTime - previousCall <= delay) {
        return;
    }
    previousCall = currentTime;
    return cb();
};

var _a, _b;
const MONITORING = Symbol('MONITORING');
const ON_SCROLL = Symbol('ON_SCROLL');
class ScrollMonitor {
    static monitorScroll() {
        if (ScrollMonitor[MONITORING]) {
            return;
        }
        window.addEventListener('scroll', ScrollMonitor[ON_SCROLL], false);
        ScrollMonitor[MONITORING] = true;
    }
}
_a = MONITORING, _b = ON_SCROLL;
ScrollMonitor.throttleDuration = 100;
ScrollMonitor.registeredAds = {};
ScrollMonitor.adCount = 0;
ScrollMonitor.subscribe = (id, element, offset = 0, onEnterViewport, onFullyEnterViewport, onExitViewport, enableByScroll) => {
    ScrollMonitor.monitorScroll();
    const existingAd = ScrollMonitor.getAdIfExists(id);
    if (existingAd) {
        if (onEnterViewport) {
            existingAd.onEnterViewport.push(onEnterViewport);
        }
        if (onFullyEnterViewport) {
            existingAd.onFullyEnterViewport.push(onFullyEnterViewport);
        }
        if (onExitViewport) {
            existingAd.onExitViewport.push(onExitViewport);
        }
        return;
    }
    const ad = {
        element,
        offset,
        inView: false,
        fullyInView: false,
        enableByScroll,
        hasViewBeenScrolled: false,
        onEnterViewport: onEnterViewport ? [onEnterViewport] : [],
        onFullyEnterViewport: onFullyEnterViewport ? [onFullyEnterViewport] : [],
        onExitViewport: onExitViewport ? [onExitViewport] : [],
    };
    ScrollMonitor.registeredAds[id] = ad;
    ScrollMonitor.evaulateCurrentViewability(ScrollMonitor.registeredAds[id], window.innerHeight);
    ++ScrollMonitor.adCount;
};
ScrollMonitor.unsubscribe = (id) => {
    if (!ScrollMonitor.registeredAds[id]) {
        return;
    }
    delete ScrollMonitor.registeredAds[id];
    --ScrollMonitor.adCount;
};
ScrollMonitor[_a] = false;
ScrollMonitor[_b] = () => throttle(() => {
    if (!ScrollMonitor.adCount) {
        return;
    }
    const windowHeight = window.innerHeight;
    Object.entries(ScrollMonitor.registeredAds).forEach(([key, ad]) => {
        ad.hasViewBeenScrolled = true;
        ScrollMonitor.evaulateCurrentViewability(ad, windowHeight);
    });
}, ScrollMonitor.throttleDuration);
ScrollMonitor.evaulateCurrentViewability = (ad, windowHeight) => {
    if (ad.enableByScroll && !ad.hasViewBeenScrolled) {
        return;
    }
    const bounding = ad.element.getBoundingClientRect();
    const inView = (bounding.top - ad.offset) <= windowHeight && (bounding.top + bounding.height) >= 0;
    const fullyInView = bounding.top >= 0 && bounding.bottom <= windowHeight;
    if (fullyInView && !ad.fullyInView) {
        ad.onFullyEnterViewport.forEach((fn) => fn());
    }
    if (inView && !ad.inView) {
        ad.onEnterViewport.forEach((fn) => fn());
    }
    if (!inView && ad.inView) {
        ad.onExitViewport.forEach((fn) => fn());
    }
    ad.inView = inView;
    ad.fullyInView = fullyInView;
};
ScrollMonitor.getAdIfExists = (id) => {
    return ScrollMonitor.registeredAds[id];
};

class GenericPlugin {
    constructor(ad) {
        if (!ad) {
            throw new Error(`An ad must be passed into the GenericPlugin class. If your Plugin inherits from GenericPlugin
		and overrides the constructor make sure you are calling "super" and that you are passing in an
		instance of an ad as the first parameter. Alternatively, you can hook into the onCreate method
		which gets called by the constructor.

		Example:
		js
		  class ExamplePlugin extends GenericPlugin {
			onCreate() {
			  console.log('Example Plugin Started Succesfully');
			}
		  }

		  // Or

		  class ExamplePlugin extends GenericPlugin {
			constructor(ad) {
			  super(ad);

			  console.log('Example Plugin Started Succesfully');
			}
		  }`);
        }
        this.ad = ad;
    }
    get name() {
        return this.constructor.name;
    }
    isEnabled(prop) {
        const value = this.ad.configuration[prop];
        if (typeof value === 'boolean') {
            return value;
        }
        return true;
    }
}

class AutoRender extends GenericPlugin {
    constructor() {
        super(...arguments);
        this.onEnterViewport = () => {
            if (!this.isEnabled('autoRender')) {
                return;
            }
            dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'AutoRender Plugin', 'Ad has entered the viewport. Calling render().');
            this.ad.render();
        };
        this.onExitViewport = () => {
            if (!this.isEnabled('autoRender')) {
                return;
            }
            dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'AutoRender Plugin', 'Ad has exited the viewport. Calling clear().');
            this.ad.clear();
        };
    }
    afterCreate() {
        if (!this.isEnabled('autoRender')) {
            return;
        }
        const { container, configuration: { renderOffset, offset, enableByScroll, clearOnExitViewport }, el: { id } } = this.ad;
        const finalOffset = renderOffset || offset || 0;
        ScrollMonitor.subscribe(id, container, finalOffset, this.onEnterViewport, undefined, clearOnExitViewport ? this.onExitViewport : undefined, enableByScroll);
        dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'AutoRender Plugin', `Ad's scroll monitor has been created.`);
    }
}

module.exports = AutoRender;
