'use strict';

var LOG_LEVELS;
(function (LOG_LEVELS) {
    LOG_LEVELS["DEBUG"] = "log";
    LOG_LEVELS["INFO"] = "info";
    LOG_LEVELS["WARN"] = "warn";
    LOG_LEVELS["ERROR"] = "error";
})(LOG_LEVELS || (LOG_LEVELS = {}));

var dispatchEvent = (id, level, creator, message, eventData) => {
    const event = new CustomEvent(`adjs:${id}:event`, {
        detail: {
            level,
            creator,
            message,
            data: eventData,
        },
    });
    window.dispatchEvent(event);
};

function insertElement(tag, attributes = {}, elementToInsertInto, html) {
    const element = document.createElement(tag);
    Object.keys(attributes).forEach((key) => {
        const value = attributes[key];
        if (typeof value === 'boolean') {
            if (!value) {
                return;
            }
            element.setAttribute(key, key);
            return;
        }
        element.setAttribute(key, value);
    });
    if (typeof html === 'string') {
        element.innerHTML = html;
    }
    elementToInsertInto.appendChild(element);
    return element;
}

function onEvent(id, fn) {
    window.addEventListener(`adjs:${id}:event`, (event) => {
        fn(event);
    });
}

function padTime(time) {
    return String(time).padStart(2, '0');
}

class GenericPlugin {
    constructor(ad) {
        if (!ad) {
            throw new Error(`An ad must be passed into the GenericPlugin class. If your Plugin inherits from GenericPlugin
		and overrides the constructor make sure you are calling "super" and that you are passing in an
		instance of an ad as the first parameter. Alternatively, you can hook into the onCreate method
		which gets called by the constructor.

		Example:
		js
		  class ExamplePlugin extends GenericPlugin {
			onCreate() {
			  console.log('Example Plugin Started Succesfully');
			}
		  }

		  // Or

		  class ExamplePlugin extends GenericPlugin {
			constructor(ad) {
			  super(ad);

			  console.log('Example Plugin Started Succesfully');
			}
		  }`);
        }
        this.ad = ad;
    }
    get name() {
        return this.constructor.name;
    }
    isEnabled(prop) {
        const value = this.ad.configuration[prop];
        if (typeof value === 'boolean') {
            return value;
        }
        return true;
    }
}

let DeveloperTools;
{
    const BASE_MESSAGE = '[ADJS]';
    const COLOR_MAP = {
        'AutoRefresh Plugin': 'blue',
        'AutoRender Plugin': 'purple',
        'DeveloperTools Plugin': 'red',
        'Responsive Plugin': 'green',
        'Sticky Plugin': 'brown',
        'DFP Network': 'gray',
    };
    const OVERLAY_STYLE = `position: absolute;
background: rgba(0, 0, 0, 0.7);
color: white;
top: 0;
left: 0;
right: 0;
bottom: 0;
overflow-y: scroll;
padding: 5px;
text-align: left;
color: yellow;
font-size: 12px;
z-index: 10000;
min-height: 200px;
`;
    const MESSAGE_STYLE = `padding: 5px 0;
color: white;
border-bottom: 1px solid grey;
`;
    DeveloperTools = class extends GenericPlugin {
        onCreate() {
            const { logging = true, overlay = true } = this.ad.configuration;
            if (logging) {
                this.initLogging();
            }
            if (overlay) {
                this.initOverlay();
            }
        }
        afterCreate() {
            this.insertMessage('created');
        }
        beforeRender() {
            dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'DeveloperTools Plugin', 'Render has been called on ad.');
        }
        onRender() {
            this.insertMessage('rendered');
            dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'DeveloperTools Plugin', 'Ad actively rendering.');
        }
        afterRender() {
            dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'DeveloperTools Plugin', 'Ad render has completed.');
        }
        beforeRefresh() {
            dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'DeveloperTools Plugin', 'Ad render has completed.');
        }
        onRefresh() {
            this.insertMessage('refreshed');
            dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'DeveloperTools Plugin', 'Ad actively refreshing.');
        }
        afterRefresh() {
            dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'DeveloperTools Plugin', 'Ad refresh has completed.');
        }
        beforeClear() {
            dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'DeveloperTools Plugin', 'Clear has been called on ad.');
        }
        onClear() {
            this.insertMessage('cleared');
            dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'DeveloperTools Plugin', 'Ad actively clearing.');
        }
        afterClear() {
            dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'DeveloperTools Plugin', 'Ad clear has completed.');
        }
        beforeDestroy() {
            dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'DeveloperTools Plugin', 'Destroy has been called on ad.');
        }
        onDestroy() {
            this.insertMessage('destroyed');
            dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'DeveloperTools Plugin', 'Ad actively destroying.');
        }
        afterDestroy() {
            dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'DeveloperTools Plugin', 'Ad destroy has completed.');
        }
        initLogging() {
            const { ad } = this;
            const { id } = ad;
            window[`ad${id}`] = ad;
            console.log(BASE_MESSAGE, 'Ad Instantiated and assigned as', `window.ad${id}`);
            onEvent(id, (event) => {
                const { level, creator, message, data = '' } = event.detail;
                const color = COLOR_MAP[creator] || 'black';
                // @ts-ignore
                console[level](`%c${BASE_MESSAGE} [${creator}] id=${id}  ${message} ${data}`, `color: ${color}; font-weight: bold;`);
            });
        }
        initOverlay() {
            const { container, el: { id }, configuration: { path, targeting } } = this.ad;
            const targetingString = targeting ? '<br />' + 'targeting = <br />' +
                Object.keys(targeting).map((key) => key + ': ' + targeting[key]).join(', <br />') : '';
            const header = `slotId: ${id} <br /> path: ${path}` + targetingString;
            this.debugOverlay = insertElement('div', { style: OVERLAY_STYLE }, container, header.concat('<hr />'));
            googletag.cmd.push(() => {
                googletag.pubads().addEventListener('slotRenderEnded', (event) => {
                    if (event.slot === this.ad.slot) {
                        this.insertMetadata(event);
                    }
                });
            });
        }
        insertMessage(baseMessage) {
            if (!this.debugOverlay) {
                return;
            }
            const currentDate = new Date();
            const hour = padTime(currentDate.getHours());
            const minutes = padTime(currentDate.getMinutes());
            const milliseconds = padTime(currentDate.getMilliseconds());
            const message = `[EVENT] ${baseMessage} at ${hour}:${minutes}:${milliseconds}`;
            insertElement('p', { style: MESSAGE_STYLE }, this.debugOverlay, message);
        }
        insertMetadata(metadata) {
            if (!this.debugOverlay) {
                return;
            }
            const message = Object.entries(metadata)
                .map(([key, value]) => `${key} = ${value}`)
                .join('<br />');
            insertElement('p', { style: MESSAGE_STYLE }, this.debugOverlay, message);
        }
    };
}
var DeveloperTools$1 = DeveloperTools;

module.exports = DeveloperTools$1;
