'use strict';

var LOG_LEVELS;
(function (LOG_LEVELS) {
    LOG_LEVELS["DEBUG"] = "log";
    LOG_LEVELS["INFO"] = "info";
    LOG_LEVELS["WARN"] = "warn";
    LOG_LEVELS["ERROR"] = "error";
})(LOG_LEVELS || (LOG_LEVELS = {}));

// Determine whether number is inbetween a set of numbers
const isBetween = (value, a, b) => value >= Math.min(a, b) && value <= Math.max(a, b);

const breakpointHandler = (sizes, breakpoints) => {
    let currentConfines = {};
    if (Array.isArray(sizes) || !breakpoints) {
        // Ad is not using responsive plugin
        return { sizesSpecified: !!sizes.length };
    }
    Object.keys(breakpoints).forEach((key) => {
        const { from, to } = breakpoints[key];
        if (isBetween(window.innerWidth, from, to)) {
            currentConfines = {
                from,
                to,
                breakpoint: key,
                sizesSpecified: !!(sizes[key] && sizes[key].length > 0),
            };
        }
    });
    return currentConfines;
};

/**
 * @param {function} callback Callback to be executed post timed debounce
 * @param {number} wait Time in Milliseconds
 */
function debounce(callback, wait) {
    let timeout;
    return (...args) => {
        const next = () => callback.apply(this, args);
        clearTimeout(timeout);
        timeout = setTimeout(next, wait);
        if (!timeout) {
            next();
        }
    };
}

var dispatchEvent = (id, level, creator, message, eventData) => {
    const event = new CustomEvent(`adjs:${id}:event`, {
        detail: {
            level,
            creator,
            message,
            data: eventData,
        },
    });
    window.dispatchEvent(event);
};

class GenericPlugin {
    constructor(ad) {
        if (!ad) {
            throw new Error(`An ad must be passed into the GenericPlugin class. If your Plugin inherits from GenericPlugin
		and overrides the constructor make sure you are calling "super" and that you are passing in an
		instance of an ad as the first parameter. Alternatively, you can hook into the onCreate method
		which gets called by the constructor.

		Example:
		js
		  class ExamplePlugin extends GenericPlugin {
			onCreate() {
			  console.log('Example Plugin Started Succesfully');
			}
		  }

		  // Or

		  class ExamplePlugin extends GenericPlugin {
			constructor(ad) {
			  super(ad);

			  console.log('Example Plugin Started Succesfully');
			}
		  }`);
        }
        this.ad = ad;
    }
    get name() {
        return this.constructor.name;
    }
    isEnabled(prop) {
        const value = this.ad.configuration[prop];
        if (typeof value === 'boolean') {
            return value;
        }
        return true;
    }
}

class Responsive extends GenericPlugin {
    constructor() {
        super(...arguments);
        this.currentConfines = {};
        this.EVENT_KEY = 'resize';
        this.THROTTLE_DURATION = 250;
    }
    beforeCreate() {
        if (this.isRefreshDisabled()) {
            dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'Responsive Plugin', `Ad refresh is disabled.`);
            return;
        }
        this.determineCurrentBreakpoint();
        this.listener = debounce(() => {
            const { from = 0, to = 1 } = this.currentConfines;
            if (isBetween(window.innerWidth, from, to)) {
                return;
            }
            this.determineCurrentBreakpoint();
            dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'Responsive Plugin', `Viewport dimensions have entered a new breakpoint. Calling refresh.`);
            this.ad.refresh();
        }, this.THROTTLE_DURATION);
        window.addEventListener(this.EVENT_KEY, this.listener);
    }
    beforeClear() {
        dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'Responsive Plugin', `Removing window resize listener for ad.`);
        window.removeEventListener(this.EVENT_KEY, this.listener);
    }
    beforeDestroy() {
        dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'Responsive Plugin', `Removing window resize listener for ad.`);
        window.removeEventListener(this.EVENT_KEY, this.listener);
    }
    determineCurrentBreakpoint() {
        const { sizes = [], breakpoints } = this.ad.configuration;
        this.currentConfines = breakpointHandler(sizes, breakpoints);
    }
    isRefreshDisabled() {
        const { configuration } = this.ad;
        return (configuration.hasOwnProperty('refreshOnBreakpoint') && !configuration.refreshOnBreakpoint);
    }
}

module.exports = Responsive;
