var AdJS = (function () {
'use strict';

const loadScript = (url, attributes = {}) => new Promise((resolve) => {
    const scriptTag = document.createElement('script');
    scriptTag.src = url;
    scriptTag.onload = () => resolve();
    Object.keys(attributes).forEach((key) => {
        const value = attributes[key];
        if (typeof value === 'boolean') {
            if (!value) {
                return;
            }
            scriptTag.setAttribute(key, key);
            return;
        }
        scriptTag.setAttribute(key, value);
    });
    document.getElementsByTagName('head')[0].appendChild(scriptTag);
});

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

function __decorate(decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
}

// Event Bus Options
var Events;
(function (Events) {
    Events["CREATE"] = "create";
    Events["REQUEST"] = "request";
    Events["RENDER"] = "render";
    Events["REFRESH"] = "refresh";
    Events["DESTROY"] = "destroy";
    Events["FREEZE"] = "freeze";
    Events["UNFREEZE"] = "unfreeze";
    Events["CLEAR"] = "clear";
})(Events || (Events = {}));
var EVENTS = Events;

function insertElement(tag, attributes = {}, elementToInsertInto, html) {
    const element = document.createElement(tag);
    Object.keys(attributes).forEach((key) => {
        const value = attributes[key];
        if (typeof value === 'boolean') {
            if (!value) {
                return;
            }
            element.setAttribute(key, key);
            return;
        }
        element.setAttribute(key, value);
    });
    if (typeof html === 'string') {
        element.innerHTML = html;
    }
    elementToInsertInto.appendChild(element);
    return element;
}

const seriallyResolvePromises = (promises) => promises.reduce((promiseChain, fn) => (promiseChain.then(() => fn())), Promise.resolve());

function uppercaseFirstLetter(word) {
    return word.charAt(0).toUpperCase() + word.slice(1);
}

let adId = 0;
function nextId() {
    return ++adId;
}
function validateSizes(configuration) {
    const { sizes, breakpoints } = configuration;
    if (!Array.isArray(sizes) && !breakpoints) {
        throw new Error('Sizes must be of type `Array` unless breakpoints have been specified');
    }
}
// Define LifeCycle Method will automatically wrap each
// lifecycle with important items such as "queue" when frozen,
// awaiting page queues and implementing vendors
function attachAsLifecycleMethod(target, propertyName, propertyDescriptor) {
    const fn = propertyDescriptor.value;
    propertyDescriptor.value = async function (...args) {
        /*
         * If the ad unit is frozen push the call into a queue that
         * can be executed later
         */
        if (this.state.frozen) {
            const boundReplayFn = this[propertyName].bind(this, ...args);
            this.actionsReceivedWhileFrozen.push(boundReplayFn);
            return;
        }
        const hookName = uppercaseFirstLetter(propertyName);
        // e.g. beforeRender, onRender, afterRender, beforeRefresh
        const beforeHookName = `before${hookName}`;
        const onHookName = `on${hookName}`;
        const afterHookName = `after${hookName}`;
        // e.g. rendering, rendered
        const executingState = `${propertyName}ing`;
        const executedState = `${propertyName}ed`;
        // Has event already been called and currently executing?
        if (this.state[executingState]) {
            return;
        }
        /*
         * Has this render method already completed succesfully? Should we
         * allow for it to be executed again?
         */
        if (this.state[executedState] && propertyName !== 'refresh') {
            return;
        }
        /*
         * Lifecycle methods are not idempotent, make sure that multiple
         * calls to a method do not execute multiple times
         */
        this.state[executingState] = true;
        /*
         * Queue up the lifecycle method's execution to ensure all page async tasks
         * have completed and that it executes in order.
         *
         * e.g. if ad.render() is called before ad.destroy(), ensure ad.render()
         *      completes before executing ad.destroy().
         */
        await this.onReady(async () => {
            this.emit(propertyName, 'before');
            await this.callPlugins(beforeHookName);
            const executionOfFn = fn.apply(this, args);
            const executionOfPlugins = this.callPlugins(onHookName);
            this.emit(propertyName, 'on');
            await Promise.all([executionOfFn, executionOfPlugins]);
            await this.callPlugins(afterHookName);
            this.state[executingState] = false;
            this.state[executedState] = true;
            this.emit(propertyName, 'after');
        });
    };
    return propertyDescriptor;
}
class Ad {
    constructor(page, el, localConfiguration) {
        this.page = page;
        this.isEmpty = true;
        this.state = {
            creating: false,
            created: false,
            rendering: false,
            rendered: false,
            refreshing: false,
            refreshed: false,
            clearing: false,
            cleared: false,
            destroying: false,
            destroyed: false,
            frozen: false,
        };
        this.actionsReceivedWhileFrozen = [];
        this.plugins = [];
        this.localVendors = [];
        this.promiseStack = Promise.resolve();
        // Event Queue
        this.events = {
            before: {},
            on: {},
            after: {},
        };
        /*
         * Add the parent pages promise chain onto each Ad instance's
         * promise chain to ensure that any async actions the parent page
         * makes (e.g. Krux) are completed before allowing a lifecycle
         * method (e.g. render) to execute
         */
        this.promiseStack = this.promiseStack.then(() => this.page.promiseStack);
        this.configuration = Object.assign(Object.assign(Object.assign({}, this.page.defaults), localConfiguration), { 
            // Targeting is the only value that should merge
            targeting: Object.assign(Object.assign({}, (this.page.defaults && this.page.defaults.targeting)), (localConfiguration && localConfiguration.targeting)) });
        this.id = nextId();
        this.container = insertElement('div', { style: 'position: relative;' }, el);
        // change this back to temp literal after rollup fix
        this.el = insertElement('div', { id: 'adjs-ad-container-' + this.id }, this.container);
        validateSizes(this.configuration);
        // Merge Locally Provided Plugins for this ad with Plugins that are specified on the Page
        const plugins = [...this.page.plugins];
        if (localConfiguration && localConfiguration.plugins) {
            plugins.push(...localConfiguration.plugins);
        }
        // Instantiate all class based plugins and reference them
        this.attachPlugins(plugins);
        const executionOfPlugins = this.callPlugins('beforeCreate');
        this.onReady(async () => {
            await executionOfPlugins;
            await Promise.all([
                this.networkInstance = this.network.createAd(this),
                this.callPlugins('onCreate'),
            ]);
            await this.callPlugins('afterCreate');
        });
    }
    get network() {
        return this.page.network;
    }
    get slot() {
        return this.networkInstance.slot;
    }
    get vendors() {
        return [
            ...this.page.vendors,
            ...this.localVendors,
        ];
    }
    // onReady will queue up additional execution calls to onReady
    // ensuring that commands called in sequence will in fact be executed
    // in sequence.
    //
    // Example:
    // ---
    //  ad.render();
    //  ad.destroy();
    //
    //  destroy must always happen after render has completed.
    //
    onReady(fn) {
        let externalResolve;
        let externalReject;
        const promiseMonitor = new Promise((resolve, reject) => {
            externalResolve = resolve;
            externalReject = reject;
        });
        this.promiseStack = this.promiseStack.then(async () => {
            try {
                await fn();
                externalResolve();
            }
            catch (e) {
                externalReject(e);
            }
        });
        return promiseMonitor;
    }
    async render() {
        await this.page.setAsActive();
        await this.networkInstance.render();
    }
    async refresh(refreshCorrelator = false) {
        await this.page.setAsActive();
        if (typeof this.networkInstance.refresh !== 'undefined') {
            await this.networkInstance.refresh(refreshCorrelator);
        }
        else {
            await this.networkInstance.destroy();
            this.networkInstance = this.network.createAd(this);
            await this.networkInstance.render();
        }
        this.state.rendered = true;
    }
    async clear() {
        await this.networkInstance.clear();
        this.state.rendered = false;
    }
    async destroy() {
        await this.networkInstance.destroy();
    }
    freeze() {
        if (this.state.frozen) {
            return;
        }
        this.emit(EVENTS.FREEZE, 'before');
        this.state.frozen = true;
        this.emit(EVENTS.FREEZE, 'on');
        this.emit(EVENTS.FREEZE, 'after');
    }
    async unfreeze(options = {}) {
        if (!this.state.frozen) {
            return;
        }
        this.emit(EVENTS.UNFREEZE, 'before');
        // unfreeze is the exception to the evented workflow because if it were
        // enqueued, it would be pushed to the end of the queue (after backlogged
        // events). Thus, leaving the ad in a limbo state. As such, we must bypass
        // the queue for this event.
        this.state.frozen = false;
        const actions = this.actionsReceivedWhileFrozen;
        this.actionsReceivedWhileFrozen = [];
        // processes backlogged events in queue on('unfreeze')
        if (options.replayEventsWhileFrozen) {
            // TODO
            this.onReady(() => seriallyResolvePromises(actions));
        }
        this.emit(EVENTS.UNFREEZE, 'on');
        this.emit(EVENTS.UNFREEZE, 'after');
    }
    on(key, fn) {
        this.attachEvent(key, fn, 'on');
    }
    before(key, fn) {
        this.attachEvent(key, fn, 'before');
    }
    after(key, fn) {
        this.attachEvent(key, fn, 'after');
    }
    // You really shouldn't await this, but it's useful to know
    // when all of the binded events have fired
    async emit(key, lifecycleTiming = 'on') {
        const events = this.events[lifecycleTiming][key];
        if (!events) {
            return;
        }
        await Promise.all(events.map((event) => event(this)));
    }
    attachEvent(key, fn, event = 'on') {
        if (!this.events[event][key]) {
            this.events[event][key] = [];
        }
        this.events[event][key].push(fn);
    }
    attachPlugins(plugins) {
        this.plugins = plugins.map((Plugin) => {
            if (typeof Plugin === 'function') {
                return new Plugin(this);
            }
            return Plugin;
        });
    }
    // TODO Figure out type
    callPlugins(hook) {
        return Promise.all(this.plugins.map(async (plugin) => {
            const hookFn = plugin[hook];
            if (typeof hookFn === 'function') {
                await hookFn.call(plugin, this);
            }
        }));
    }
}
__decorate([
    attachAsLifecycleMethod
], Ad.prototype, "render", null);
__decorate([
    attachAsLifecycleMethod
], Ad.prototype, "refresh", null);
__decorate([
    attachAsLifecycleMethod
], Ad.prototype, "clear", null);
__decorate([
    attachAsLifecycleMethod
], Ad.prototype, "destroy", null);

class Page {
    constructor(network, providedConfiguration = {}) {
        this.network = network;
        this.ads = [];
        this.promiseStack = Promise.resolve();
        this.plugins = [];
        this.vendors = [];
        this.defaults = {};
        const { defaults, plugins, vendors } = providedConfiguration;
        if (defaults) {
            this.defaults = defaults;
        }
        if (plugins) {
            this.plugins = plugins;
        }
        if (vendors) {
            this.vendors = vendors;
        }
        AdJS.pages.push(this);
    }
    Ad(el, options) {
        return this.createAd(el, options);
    }
    createAd(el, options) {
        const ad = new Ad(this, el, options);
        this.ads.push(ad);
        return ad;
    }
    get correlatorId() {
        const { correlatorId } = this.ads[0];
        return correlatorId;
    }
    async setAsActive() {
        if (!this.correlatorId || AdJS.activeCorrelatorId === this.correlatorId) {
            return;
        }
        this.ads.forEach((ad) => {
            ad.clear();
        });
        await this.network.resetCorrelator();
    }
}

const isServer = () => typeof window === 'undefined';

const LOADED_MODULES = {
    Plugins: {},
    Networks: {},
    Vendors: {},
};
const _ADJS = isServer() ? LOADED_MODULES : window._ADJS = window._ADJS || LOADED_MODULES;
_ADJS.Plugins = _ADJS.Plugins || LOADED_MODULES.Plugins;
_ADJS.Vendors = _ADJS.Vendors || LOADED_MODULES.Vendors;
_ADJS.Networks = _ADJS.Networks || LOADED_MODULES.Networks;
const PluginsHandler = {
    get(plugins, property) {
        const cleanedProperty = String(property);
        if (isServer()) {
            if (typeof property === 'symbol') {
                return {};
            }
            throw new Error(`Using the Networks or Plugins property on AdJS is only available when installing via script.
If you are compiling the AdJS library locally within your project, use require to
specify the plugin directly.

Example:
  import DFP from 'adjs/networks/DFP';
  import AutoRender from 'adjs/plugins/AutoRender';

  new AdJS.Page(DFP, {
	plugins: [
	  AutoRender,
	],
  });`);
        }
        if (!plugins[cleanedProperty]) {
            throw new Error(`The Plugin or Network has not been included in your bundle.
Please manually include the script tag associated with this plugin or network.

Example:
  <script src="https://cdn.adjs.dev/core.min.js"></script>
  <script src="https://cdn.adjs.dev/DFP.min.js"></script>
  <script src="https://cdn.adjs.dev/AutoRender.min.js"></script>

  <script>
	new AdJS.Page(AdJS.Networks.DFP, {
	  plugins: [
		AdJS.Plugins.AutoRender,
	  ],
	});
  </script>`);
        }
        return plugins[cleanedProperty];
    },
};
const Plugins = new Proxy(_ADJS.Plugins, PluginsHandler);
const Networks = new Proxy(_ADJS.Networks, PluginsHandler);
const Vendors = new Proxy(_ADJS.Vendors, PluginsHandler);
const pages = [];
const Debug = () => {
    loadScript('https://unpkg.com/adjs@latest/umd/debug.production.min.js', { async: true, defer: true });
    return 'Loading developer tools...';
};
const AdJS = {
    Page,
    pages,
    Bucket: Page,
    Plugins,
    Networks,
    Vendors,
    DEBUG: Debug,
    activeCorrelatorId: null,
    VERSION: '2.0.0-beta.33',
};
if (!isServer()) {
    window[`AdJS`] = AdJS;
}

return AdJS;

}());
//# sourceMappingURL=core.development.js.map
this._ADJS = this._ADJS || {};
this._ADJS.Networks = this._ADJS.Networks || {};
this._ADJS.Networks.DFP = (function () {
'use strict';

var LOG_LEVELS;
(function (LOG_LEVELS) {
    LOG_LEVELS["DEBUG"] = "log";
    LOG_LEVELS["INFO"] = "info";
    LOG_LEVELS["WARN"] = "warn";
    LOG_LEVELS["ERROR"] = "error";
})(LOG_LEVELS || (LOG_LEVELS = {}));

// Determine whether number is inbetween a set of numbers
const isBetween = (value, a, b) => value >= Math.min(a, b) && value <= Math.max(a, b);

const breakpointHandler = (sizes, breakpoints) => {
    let currentConfines = {};
    if (Array.isArray(sizes) || !breakpoints) {
        // Ad is not using responsive plugin
        return { sizesSpecified: !!sizes.length };
    }
    Object.keys(breakpoints).forEach((key) => {
        const { from, to } = breakpoints[key];
        if (isBetween(window.innerWidth, from, to)) {
            currentConfines = {
                from,
                to,
                breakpoint: key,
                sizesSpecified: !!(sizes[key] && sizes[key].length > 0),
            };
        }
    });
    return currentConfines;
};

var dispatchEvent = (id, level, creator, message, eventData) => {
    const event = new CustomEvent(`adjs:${id}:event`, {
        detail: {
            level,
            creator,
            message,
            data: eventData,
        },
    });
    window.dispatchEvent(event);
};

const loadScript = (url, attributes = {}) => new Promise((resolve) => {
    const scriptTag = document.createElement('script');
    scriptTag.src = url;
    scriptTag.onload = () => resolve();
    Object.keys(attributes).forEach((key) => {
        const value = attributes[key];
        if (typeof value === 'boolean') {
            if (!value) {
                return;
            }
            scriptTag.setAttribute(key, key);
            return;
        }
        scriptTag.setAttribute(key, value);
    });
    document.getElementsByTagName('head')[0].appendChild(scriptTag);
});

class DfpAd {
    constructor(ad) {
        this.ad = ad;
        DoubleClickForPublishers.prepare();
        const { el: { id }, configuration: { sizes, targeting, path, breakpoints, collapsible }, } = ad;
        if (!id) {
            throw new Error('Ad does not have an id');
        }
        if (!sizes) {
            throw new Error('Sizes must be defined.');
        }
        if (!path) {
            throw new Error('Ad Path must be defined.');
        }
        this.id = id;
        this.sizes = sizes;
        this.breakpoints = breakpoints;
        googletag.cmd.push(() => {
            this.slot = googletag.defineSlot(path, Array.isArray(sizes) ? sizes : [0, 0], this.id);
            if (collapsible) {
                this.slot.setCollapseEmptyDiv(true, true);
            }
            if (targeting) {
                Object.entries(targeting)
                    .forEach(([key, value]) => {
                    this.slot.setTargeting(key, value);
                });
                dispatchEvent(Number(this.id.substring(this.id.length, this.id.length - 1)), LOG_LEVELS.INFO, 'DFP Network', 'Targeting detected for ad. Adding to configuration.');
            }
            if (!Array.isArray(sizes) && breakpoints) {
                const sizing = googletag.sizeMapping();
                Object.entries(breakpoints).map(([device, dimensions]) => {
                    sizing.addSize([dimensions.from, 1], sizes[device] || []);
                });
                this.slot.defineSizeMapping(sizing.build());
            }
            this.slot.addService(googletag.pubads());
            googletag.display(this.id);
        });
    }
    render() {
        return this.refresh();
    }
    clear() {
        return new Promise((resolve) => {
            googletag.cmd.push(() => {
                const { slot } = this;
                googletag.pubads().clear([slot]);
                this.ad.isEmpty = true;
                resolve();
            });
        });
    }
    // TODO: Clean up event listeners
    refresh(resetCorrelator = false) {
        return new Promise((resolve) => {
            if (!breakpointHandler(this.sizes, this.breakpoints).sizesSpecified) {
                this.clear().then(resolve);
                return;
            }
            googletag.cmd.push(() => {
                const { slot } = this;
                googletag.pubads().addEventListener('slotRenderEnded', (event) => {
                    if (event.slot === slot) {
                        this.ad.isEmpty = event.isEmpty;
                        dispatchEvent(Number(this.id.substring(this.id.length, this.id.length - 1)), LOG_LEVELS.INFO, 'DFP Network', 'Ad slot has been rendered.');
                        resolve();
                    }
                });
                googletag.pubads().refresh([slot], { changeCorrelator: resetCorrelator });
                if (!slot.getContentUrl()) {
                    this.ad.isEmpty = true;
                    dispatchEvent(Number(this.id.substring(this.id.length, this.id.length - 1)), LOG_LEVELS.WARN, 'DFP Network', 'Ad sizes missing. Bypassing ad.');
                    resolve();
                }
            });
        });
    }
    // Cannot undo this action
    destroy() {
        return new Promise((resolve) => {
            googletag.cmd.push(() => {
                const { slot } = this;
                googletag.destroySlots([slot]);
                this.ad.isEmpty = true;
                resolve();
            });
        });
    }
}
function loadGPT() {
    if (window.googletag && window.googletag.apiReady) {
        return;
    }
    loadScript('https://securepubads.g.doubleclick.net/tag/js/gpt.js', { async: true });
    window.googletag = window.googletag || {};
    window.googletag.cmd = window.googletag.cmd || [];
    window.googletag.cmd.push(() => {
        googletag.pubads().disableInitialLoad();
        googletag.pubads().enableAsyncRendering();
        googletag.enableServices();
    });
}
const DoubleClickForPublishers = {
    name: 'DoubleClick for Publishers',
    loaded: false,
    createAd(ad) {
        return new DfpAd(ad);
    },
    async prepare() {
        if (this.loaded) {
            return;
        }
        loadGPT();
        this.loaded = true;
    },
    async resetCorrelator() {
        googletag.pubads().updateCorrelator();
    },
};

return DoubleClickForPublishers;

}());
//# sourceMappingURL=networks.dfp.development.js.map
this._ADJS = this._ADJS || {};
this._ADJS.Networks = this._ADJS.Networks || {};
this._ADJS.Networks.Mock = (function () {
'use strict';

const MockAdInstance = {
    async render() {
        return new Promise((resolve) => {
            setTimeout(resolve, 400);
        });
    },
    async destroy() {
    },
    async clear() {
    },
};
const MockAdNetwork = {
    name: 'Mock Network',
    createAd(ad) {
        return MockAdInstance;
    },
    async resetCorrelator() {
        return Promise.resolve();
    },
};

return MockAdNetwork;

}());
//# sourceMappingURL=networks.mock.development.js.map
this._ADJS = this._ADJS || {};
this._ADJS.Networks = this._ADJS.Networks || {};
this._ADJS.Networks.Noop = (function () {
'use strict';

const NoopAd = {
    async render() { },
    async destroy() { },
    async refresh() { },
    async clear() { },
};
const Noop = {
    name: 'The Does Absolutely Nothing Network',
    createAd(ad) {
        return NoopAd;
    },
    async resetCorrelator() { },
};

return Noop;

}());
//# sourceMappingURL=networks.noop.development.js.map
this._ADJS = this._ADJS || {};
this._ADJS.Plugins = this._ADJS.Plugins || {};
this._ADJS.Plugins.AutoRefresh = (function () {
'use strict';

var LOG_LEVELS;
(function (LOG_LEVELS) {
    LOG_LEVELS["DEBUG"] = "log";
    LOG_LEVELS["INFO"] = "info";
    LOG_LEVELS["WARN"] = "warn";
    LOG_LEVELS["ERROR"] = "error";
})(LOG_LEVELS || (LOG_LEVELS = {}));

var dispatchEvent = (id, level, creator, message, eventData) => {
    const event = new CustomEvent(`adjs:${id}:event`, {
        detail: {
            level,
            creator,
            message,
            data: eventData,
        },
    });
    window.dispatchEvent(event);
};

/**
 * @param {function} cb Callback to be executed post timed throttle
 * @param {number} customDelay Time in Milliseconds
 */
let previousCall = 0;
var throttle = (cb, customDelay) => {
    const delay = customDelay || 200;
    const currentTime = new Date().getTime();
    if (currentTime - previousCall <= delay) {
        return;
    }
    previousCall = currentTime;
    return cb();
};

var _a, _b;
const MONITORING = Symbol('MONITORING');
const ON_SCROLL = Symbol('ON_SCROLL');
class ScrollMonitor {
    static monitorScroll() {
        if (ScrollMonitor[MONITORING]) {
            return;
        }
        window.addEventListener('scroll', ScrollMonitor[ON_SCROLL], false);
        ScrollMonitor[MONITORING] = true;
    }
}
_a = MONITORING, _b = ON_SCROLL;
ScrollMonitor.throttleDuration = 100;
ScrollMonitor.registeredAds = {};
ScrollMonitor.adCount = 0;
ScrollMonitor.subscribe = (id, element, offset = 0, onEnterViewport, onFullyEnterViewport, onExitViewport, enableByScroll) => {
    ScrollMonitor.monitorScroll();
    const existingAd = ScrollMonitor.getAdIfExists(id);
    if (existingAd) {
        if (onEnterViewport) {
            existingAd.onEnterViewport.push(onEnterViewport);
        }
        if (onFullyEnterViewport) {
            existingAd.onFullyEnterViewport.push(onFullyEnterViewport);
        }
        if (onExitViewport) {
            existingAd.onExitViewport.push(onExitViewport);
        }
        return;
    }
    const ad = {
        element,
        offset,
        inView: false,
        fullyInView: false,
        enableByScroll,
        hasViewBeenScrolled: false,
        onEnterViewport: onEnterViewport ? [onEnterViewport] : [],
        onFullyEnterViewport: onFullyEnterViewport ? [onFullyEnterViewport] : [],
        onExitViewport: onExitViewport ? [onExitViewport] : [],
    };
    ScrollMonitor.registeredAds[id] = ad;
    ScrollMonitor.evaulateCurrentViewability(ScrollMonitor.registeredAds[id], window.innerHeight);
    ++ScrollMonitor.adCount;
};
ScrollMonitor.unsubscribe = (id) => {
    if (!ScrollMonitor.registeredAds[id]) {
        return;
    }
    delete ScrollMonitor.registeredAds[id];
    --ScrollMonitor.adCount;
};
ScrollMonitor[_a] = false;
ScrollMonitor[_b] = () => throttle(() => {
    if (!ScrollMonitor.adCount) {
        return;
    }
    const windowHeight = window.innerHeight;
    Object.entries(ScrollMonitor.registeredAds).forEach(([key, ad]) => {
        ad.hasViewBeenScrolled = true;
        ScrollMonitor.evaulateCurrentViewability(ad, windowHeight);
    });
}, ScrollMonitor.throttleDuration);
ScrollMonitor.evaulateCurrentViewability = (ad, windowHeight) => {
    if (ad.enableByScroll && !ad.hasViewBeenScrolled) {
        return;
    }
    const bounding = ad.element.getBoundingClientRect();
    const inView = (bounding.top - ad.offset) <= windowHeight && (bounding.top + bounding.height) >= 0;
    const fullyInView = bounding.top >= 0 && bounding.bottom <= windowHeight;
    if (fullyInView && !ad.fullyInView) {
        ad.onFullyEnterViewport.forEach((fn) => fn());
    }
    if (inView && !ad.inView) {
        ad.onEnterViewport.forEach((fn) => fn());
    }
    if (!inView && ad.inView) {
        ad.onExitViewport.forEach((fn) => fn());
    }
    ad.inView = inView;
    ad.fullyInView = fullyInView;
};
ScrollMonitor.getAdIfExists = (id) => {
    return ScrollMonitor.registeredAds[id];
};

class GenericPlugin {
    constructor(ad) {
        if (!ad) {
            throw new Error(`An ad must be passed into the GenericPlugin class. If your Plugin inherits from GenericPlugin
		and overrides the constructor make sure you are calling "super" and that you are passing in an
		instance of an ad as the first parameter. Alternatively, you can hook into the onCreate method
		which gets called by the constructor.

		Example:
		js
		  class ExamplePlugin extends GenericPlugin {
			onCreate() {
			  console.log('Example Plugin Started Succesfully');
			}
		  }

		  // Or

		  class ExamplePlugin extends GenericPlugin {
			constructor(ad) {
			  super(ad);

			  console.log('Example Plugin Started Succesfully');
			}
		  }`);
        }
        this.ad = ad;
    }
    get name() {
        return this.constructor.name;
    }
    isEnabled(prop) {
        const value = this.ad.configuration[prop];
        if (typeof value === 'boolean') {
            return value;
        }
        return true;
    }
}

const ONE_SECOND = 1000;
class AutoRefresh extends GenericPlugin {
    constructor() {
        super(...arguments);
        this.timeInView = 0;
        this.isRefreshing = false;
        // Number of times the ad refreshed.
        this.refreshCount = 0;
        this.markAsInView = () => {
            if (this.timerReference) {
                return;
            }
            const { refreshRateInSeconds = 30, refreshLimit = null, refreshResetsCorrelator = false } = this.ad.configuration;
            dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'AutoRefresh Plugin', `Ad is in view. Beginning timer for ${refreshRateInSeconds}.`);
            this.timerReference = window.setInterval(async () => {
                this.timeInView += 1;
                // Determine whether the ad should refresh
                const underRefreshLimit = (refreshLimit === null || this.refreshCount < refreshLimit);
                if (!this.isRefreshing && this.timeInView >= refreshRateInSeconds && underRefreshLimit) {
                    this.isRefreshing = true;
                    this.refreshCount += 1;
                    // Once we hit the limit, clear the interval.
                    if (this.refreshCount === refreshLimit) {
                        clearInterval(this.timerReference);
                    }
                    dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'AutoRefresh Plugin', `Ad viewability met. Refreshing Ad and resetting timer.`);
                    await this.ad.refresh(refreshResetsCorrelator);
                    this.timeInView = 0;
                    this.isRefreshing = false;
                }
            }, ONE_SECOND);
        };
        this.markAsOutOfView = () => {
            if (!this.timerReference) {
                return;
            }
            dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'AutoRefresh Plugin', 'Ad is no longer in view.');
            clearInterval(this.timerReference);
            this.timerReference = undefined;
        };
    }
    afterRender() {
        dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'AutoRefresh Plugin', 'Adding monitoring for ad.');
        this.startMonitoringViewability();
        dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'AutoRefresh Plugin', 'Ad viewability is being monitored.');
    }
    beforeClear() {
        ScrollMonitor.unsubscribe(this.ad.el.id);
        dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'AutoRefresh Plugin', 'Ad viewability monitor has been removed.');
    }
    beforeDestroy() {
        ScrollMonitor.unsubscribe(this.ad.el.id);
        dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'AutoRefresh Plugin', 'Ad viewability monitor has been removed.');
    }
    startMonitoringViewability() {
        const { container, configuration: { offset = 0 }, el } = this.ad;
        ScrollMonitor.subscribe(el.id, container, offset, undefined, this.markAsInView, this.markAsOutOfView);
        this.timeInView = 0;
    }
}

return AutoRefresh;

}());
//# sourceMappingURL=plugins.autorefresh.development.js.map
this._ADJS = this._ADJS || {};
this._ADJS.Plugins = this._ADJS.Plugins || {};
this._ADJS.Plugins.AutoRender = (function () {
'use strict';

var LOG_LEVELS;
(function (LOG_LEVELS) {
    LOG_LEVELS["DEBUG"] = "log";
    LOG_LEVELS["INFO"] = "info";
    LOG_LEVELS["WARN"] = "warn";
    LOG_LEVELS["ERROR"] = "error";
})(LOG_LEVELS || (LOG_LEVELS = {}));

var dispatchEvent = (id, level, creator, message, eventData) => {
    const event = new CustomEvent(`adjs:${id}:event`, {
        detail: {
            level,
            creator,
            message,
            data: eventData,
        },
    });
    window.dispatchEvent(event);
};

/**
 * @param {function} cb Callback to be executed post timed throttle
 * @param {number} customDelay Time in Milliseconds
 */
let previousCall = 0;
var throttle = (cb, customDelay) => {
    const delay = customDelay || 200;
    const currentTime = new Date().getTime();
    if (currentTime - previousCall <= delay) {
        return;
    }
    previousCall = currentTime;
    return cb();
};

var _a, _b;
const MONITORING = Symbol('MONITORING');
const ON_SCROLL = Symbol('ON_SCROLL');
class ScrollMonitor {
    static monitorScroll() {
        if (ScrollMonitor[MONITORING]) {
            return;
        }
        window.addEventListener('scroll', ScrollMonitor[ON_SCROLL], false);
        ScrollMonitor[MONITORING] = true;
    }
}
_a = MONITORING, _b = ON_SCROLL;
ScrollMonitor.throttleDuration = 100;
ScrollMonitor.registeredAds = {};
ScrollMonitor.adCount = 0;
ScrollMonitor.subscribe = (id, element, offset = 0, onEnterViewport, onFullyEnterViewport, onExitViewport, enableByScroll) => {
    ScrollMonitor.monitorScroll();
    const existingAd = ScrollMonitor.getAdIfExists(id);
    if (existingAd) {
        if (onEnterViewport) {
            existingAd.onEnterViewport.push(onEnterViewport);
        }
        if (onFullyEnterViewport) {
            existingAd.onFullyEnterViewport.push(onFullyEnterViewport);
        }
        if (onExitViewport) {
            existingAd.onExitViewport.push(onExitViewport);
        }
        return;
    }
    const ad = {
        element,
        offset,
        inView: false,
        fullyInView: false,
        enableByScroll,
        hasViewBeenScrolled: false,
        onEnterViewport: onEnterViewport ? [onEnterViewport] : [],
        onFullyEnterViewport: onFullyEnterViewport ? [onFullyEnterViewport] : [],
        onExitViewport: onExitViewport ? [onExitViewport] : [],
    };
    ScrollMonitor.registeredAds[id] = ad;
    ScrollMonitor.evaulateCurrentViewability(ScrollMonitor.registeredAds[id], window.innerHeight);
    ++ScrollMonitor.adCount;
};
ScrollMonitor.unsubscribe = (id) => {
    if (!ScrollMonitor.registeredAds[id]) {
        return;
    }
    delete ScrollMonitor.registeredAds[id];
    --ScrollMonitor.adCount;
};
ScrollMonitor[_a] = false;
ScrollMonitor[_b] = () => throttle(() => {
    if (!ScrollMonitor.adCount) {
        return;
    }
    const windowHeight = window.innerHeight;
    Object.entries(ScrollMonitor.registeredAds).forEach(([key, ad]) => {
        ad.hasViewBeenScrolled = true;
        ScrollMonitor.evaulateCurrentViewability(ad, windowHeight);
    });
}, ScrollMonitor.throttleDuration);
ScrollMonitor.evaulateCurrentViewability = (ad, windowHeight) => {
    if (ad.enableByScroll && !ad.hasViewBeenScrolled) {
        return;
    }
    const bounding = ad.element.getBoundingClientRect();
    const inView = (bounding.top - ad.offset) <= windowHeight && (bounding.top + bounding.height) >= 0;
    const fullyInView = bounding.top >= 0 && bounding.bottom <= windowHeight;
    if (fullyInView && !ad.fullyInView) {
        ad.onFullyEnterViewport.forEach((fn) => fn());
    }
    if (inView && !ad.inView) {
        ad.onEnterViewport.forEach((fn) => fn());
    }
    if (!inView && ad.inView) {
        ad.onExitViewport.forEach((fn) => fn());
    }
    ad.inView = inView;
    ad.fullyInView = fullyInView;
};
ScrollMonitor.getAdIfExists = (id) => {
    return ScrollMonitor.registeredAds[id];
};

class GenericPlugin {
    constructor(ad) {
        if (!ad) {
            throw new Error(`An ad must be passed into the GenericPlugin class. If your Plugin inherits from GenericPlugin
		and overrides the constructor make sure you are calling "super" and that you are passing in an
		instance of an ad as the first parameter. Alternatively, you can hook into the onCreate method
		which gets called by the constructor.

		Example:
		js
		  class ExamplePlugin extends GenericPlugin {
			onCreate() {
			  console.log('Example Plugin Started Succesfully');
			}
		  }

		  // Or

		  class ExamplePlugin extends GenericPlugin {
			constructor(ad) {
			  super(ad);

			  console.log('Example Plugin Started Succesfully');
			}
		  }`);
        }
        this.ad = ad;
    }
    get name() {
        return this.constructor.name;
    }
    isEnabled(prop) {
        const value = this.ad.configuration[prop];
        if (typeof value === 'boolean') {
            return value;
        }
        return true;
    }
}

class AutoRender extends GenericPlugin {
    constructor() {
        super(...arguments);
        this.onEnterViewport = () => {
            if (!this.isEnabled('autoRender')) {
                return;
            }
            dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'AutoRender Plugin', 'Ad has entered the viewport. Calling render().');
            this.ad.render();
        };
        this.onExitViewport = () => {
            if (!this.isEnabled('autoRender')) {
                return;
            }
            dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'AutoRender Plugin', 'Ad has exited the viewport. Calling clear().');
            this.ad.clear();
        };
    }
    afterCreate() {
        if (!this.isEnabled('autoRender')) {
            return;
        }
        const { container, configuration: { renderOffset, offset, enableByScroll, clearOnExitViewport }, el: { id } } = this.ad;
        const finalOffset = renderOffset || offset || 0;
        ScrollMonitor.subscribe(id, container, finalOffset, this.onEnterViewport, undefined, clearOnExitViewport ? this.onExitViewport : undefined, enableByScroll);
        dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'AutoRender Plugin', `Ad's scroll monitor has been created.`);
    }
}

return AutoRender;

}());
//# sourceMappingURL=plugins.autorender.development.js.map
this._ADJS = this._ADJS || {};
this._ADJS.Plugins = this._ADJS.Plugins || {};
this._ADJS.Plugins.DeveloperTools = (function () {
'use strict';

var LOG_LEVELS;
(function (LOG_LEVELS) {
    LOG_LEVELS["DEBUG"] = "log";
    LOG_LEVELS["INFO"] = "info";
    LOG_LEVELS["WARN"] = "warn";
    LOG_LEVELS["ERROR"] = "error";
})(LOG_LEVELS || (LOG_LEVELS = {}));

var dispatchEvent = (id, level, creator, message, eventData) => {
    const event = new CustomEvent(`adjs:${id}:event`, {
        detail: {
            level,
            creator,
            message,
            data: eventData,
        },
    });
    window.dispatchEvent(event);
};

function insertElement(tag, attributes = {}, elementToInsertInto, html) {
    const element = document.createElement(tag);
    Object.keys(attributes).forEach((key) => {
        const value = attributes[key];
        if (typeof value === 'boolean') {
            if (!value) {
                return;
            }
            element.setAttribute(key, key);
            return;
        }
        element.setAttribute(key, value);
    });
    if (typeof html === 'string') {
        element.innerHTML = html;
    }
    elementToInsertInto.appendChild(element);
    return element;
}

function onEvent(id, fn) {
    window.addEventListener(`adjs:${id}:event`, (event) => {
        fn(event);
    });
}

function padTime(time) {
    return String(time).padStart(2, '0');
}

class GenericPlugin {
    constructor(ad) {
        if (!ad) {
            throw new Error(`An ad must be passed into the GenericPlugin class. If your Plugin inherits from GenericPlugin
		and overrides the constructor make sure you are calling "super" and that you are passing in an
		instance of an ad as the first parameter. Alternatively, you can hook into the onCreate method
		which gets called by the constructor.

		Example:
		js
		  class ExamplePlugin extends GenericPlugin {
			onCreate() {
			  console.log('Example Plugin Started Succesfully');
			}
		  }

		  // Or

		  class ExamplePlugin extends GenericPlugin {
			constructor(ad) {
			  super(ad);

			  console.log('Example Plugin Started Succesfully');
			}
		  }`);
        }
        this.ad = ad;
    }
    get name() {
        return this.constructor.name;
    }
    isEnabled(prop) {
        const value = this.ad.configuration[prop];
        if (typeof value === 'boolean') {
            return value;
        }
        return true;
    }
}

let DeveloperTools;
{
    const BASE_MESSAGE = '[ADJS]';
    const COLOR_MAP = {
        'AutoRefresh Plugin': 'blue',
        'AutoRender Plugin': 'purple',
        'DeveloperTools Plugin': 'red',
        'Responsive Plugin': 'green',
        'Sticky Plugin': 'brown',
        'DFP Network': 'gray',
    };
    const OVERLAY_STYLE = `position: absolute;
background: rgba(0, 0, 0, 0.7);
color: white;
top: 0;
left: 0;
right: 0;
bottom: 0;
overflow-y: scroll;
padding: 5px;
text-align: left;
color: yellow;
font-size: 12px;
z-index: 10000;
min-height: 200px;
`;
    const MESSAGE_STYLE = `padding: 5px 0;
color: white;
border-bottom: 1px solid grey;
`;
    DeveloperTools = class extends GenericPlugin {
        onCreate() {
            const { logging = true, overlay = true } = this.ad.configuration;
            if (logging) {
                this.initLogging();
            }
            if (overlay) {
                this.initOverlay();
            }
        }
        afterCreate() {
            this.insertMessage('created');
        }
        beforeRender() {
            dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'DeveloperTools Plugin', 'Render has been called on ad.');
        }
        onRender() {
            this.insertMessage('rendered');
            dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'DeveloperTools Plugin', 'Ad actively rendering.');
        }
        afterRender() {
            dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'DeveloperTools Plugin', 'Ad render has completed.');
        }
        beforeRefresh() {
            dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'DeveloperTools Plugin', 'Ad render has completed.');
        }
        onRefresh() {
            this.insertMessage('refreshed');
            dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'DeveloperTools Plugin', 'Ad actively refreshing.');
        }
        afterRefresh() {
            dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'DeveloperTools Plugin', 'Ad refresh has completed.');
        }
        beforeClear() {
            dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'DeveloperTools Plugin', 'Clear has been called on ad.');
        }
        onClear() {
            this.insertMessage('cleared');
            dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'DeveloperTools Plugin', 'Ad actively clearing.');
        }
        afterClear() {
            dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'DeveloperTools Plugin', 'Ad clear has completed.');
        }
        beforeDestroy() {
            dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'DeveloperTools Plugin', 'Destroy has been called on ad.');
        }
        onDestroy() {
            this.insertMessage('destroyed');
            dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'DeveloperTools Plugin', 'Ad actively destroying.');
        }
        afterDestroy() {
            dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'DeveloperTools Plugin', 'Ad destroy has completed.');
        }
        initLogging() {
            const { ad } = this;
            const { id } = ad;
            window[`ad${id}`] = ad;
            console.log(BASE_MESSAGE, 'Ad Instantiated and assigned as', `window.ad${id}`);
            onEvent(id, (event) => {
                const { level, creator, message, data = '' } = event.detail;
                const color = COLOR_MAP[creator] || 'black';
                // @ts-ignore
                console[level](`%c${BASE_MESSAGE} [${creator}] id=${id}  ${message} ${data}`, `color: ${color}; font-weight: bold;`);
            });
        }
        initOverlay() {
            const { container, el: { id }, configuration: { path, targeting } } = this.ad;
            const targetingString = targeting ? '<br />' + 'targeting = <br />' +
                Object.keys(targeting).map((key) => key + ': ' + targeting[key]).join(', <br />') : '';
            const header = `slotId: ${id} <br /> path: ${path}` + targetingString;
            this.debugOverlay = insertElement('div', { style: OVERLAY_STYLE }, container, header.concat('<hr />'));
            googletag.cmd.push(() => {
                googletag.pubads().addEventListener('slotRenderEnded', (event) => {
                    if (event.slot === this.ad.slot) {
                        this.insertMetadata(event);
                    }
                });
            });
        }
        insertMessage(baseMessage) {
            if (!this.debugOverlay) {
                return;
            }
            const currentDate = new Date();
            const hour = padTime(currentDate.getHours());
            const minutes = padTime(currentDate.getMinutes());
            const milliseconds = padTime(currentDate.getMilliseconds());
            const message = `[EVENT] ${baseMessage} at ${hour}:${minutes}:${milliseconds}`;
            insertElement('p', { style: MESSAGE_STYLE }, this.debugOverlay, message);
        }
        insertMetadata(metadata) {
            if (!this.debugOverlay) {
                return;
            }
            const message = Object.entries(metadata)
                .map(([key, value]) => `${key} = ${value}`)
                .join('<br />');
            insertElement('p', { style: MESSAGE_STYLE }, this.debugOverlay, message);
        }
    };
}
var DeveloperTools$1 = DeveloperTools;

return DeveloperTools$1;

}());
//# sourceMappingURL=plugins.developertools.development.js.map
this._ADJS = this._ADJS || {};
this._ADJS.Plugins = this._ADJS.Plugins || {};
this._ADJS.Plugins.GenericPlugin = (function () {
'use strict';

class GenericPlugin {
    constructor(ad) {
        if (!ad) {
            throw new Error(`An ad must be passed into the GenericPlugin class. If your Plugin inherits from GenericPlugin
		and overrides the constructor make sure you are calling "super" and that you are passing in an
		instance of an ad as the first parameter. Alternatively, you can hook into the onCreate method
		which gets called by the constructor.

		Example:
		js
		  class ExamplePlugin extends GenericPlugin {
			onCreate() {
			  console.log('Example Plugin Started Succesfully');
			}
		  }

		  // Or

		  class ExamplePlugin extends GenericPlugin {
			constructor(ad) {
			  super(ad);

			  console.log('Example Plugin Started Succesfully');
			}
		  }`);
        }
        this.ad = ad;
    }
    get name() {
        return this.constructor.name;
    }
    isEnabled(prop) {
        const value = this.ad.configuration[prop];
        if (typeof value === 'boolean') {
            return value;
        }
        return true;
    }
}

return GenericPlugin;

}());
//# sourceMappingURL=plugins.genericplugin.development.js.map
this._ADJS = this._ADJS || {};
this._ADJS.Plugins = this._ADJS.Plugins || {};
this._ADJS.Plugins.Responsive = (function () {
'use strict';

var LOG_LEVELS;
(function (LOG_LEVELS) {
    LOG_LEVELS["DEBUG"] = "log";
    LOG_LEVELS["INFO"] = "info";
    LOG_LEVELS["WARN"] = "warn";
    LOG_LEVELS["ERROR"] = "error";
})(LOG_LEVELS || (LOG_LEVELS = {}));

// Determine whether number is inbetween a set of numbers
const isBetween = (value, a, b) => value >= Math.min(a, b) && value <= Math.max(a, b);

const breakpointHandler = (sizes, breakpoints) => {
    let currentConfines = {};
    if (Array.isArray(sizes) || !breakpoints) {
        // Ad is not using responsive plugin
        return { sizesSpecified: !!sizes.length };
    }
    Object.keys(breakpoints).forEach((key) => {
        const { from, to } = breakpoints[key];
        if (isBetween(window.innerWidth, from, to)) {
            currentConfines = {
                from,
                to,
                breakpoint: key,
                sizesSpecified: !!(sizes[key] && sizes[key].length > 0),
            };
        }
    });
    return currentConfines;
};

/**
 * @param {function} callback Callback to be executed post timed debounce
 * @param {number} wait Time in Milliseconds
 */
function debounce(callback, wait) {
    let timeout;
    return (...args) => {
        const next = () => callback.apply(this, args);
        clearTimeout(timeout);
        timeout = setTimeout(next, wait);
        if (!timeout) {
            next();
        }
    };
}

var dispatchEvent = (id, level, creator, message, eventData) => {
    const event = new CustomEvent(`adjs:${id}:event`, {
        detail: {
            level,
            creator,
            message,
            data: eventData,
        },
    });
    window.dispatchEvent(event);
};

class GenericPlugin {
    constructor(ad) {
        if (!ad) {
            throw new Error(`An ad must be passed into the GenericPlugin class. If your Plugin inherits from GenericPlugin
		and overrides the constructor make sure you are calling "super" and that you are passing in an
		instance of an ad as the first parameter. Alternatively, you can hook into the onCreate method
		which gets called by the constructor.

		Example:
		js
		  class ExamplePlugin extends GenericPlugin {
			onCreate() {
			  console.log('Example Plugin Started Succesfully');
			}
		  }

		  // Or

		  class ExamplePlugin extends GenericPlugin {
			constructor(ad) {
			  super(ad);

			  console.log('Example Plugin Started Succesfully');
			}
		  }`);
        }
        this.ad = ad;
    }
    get name() {
        return this.constructor.name;
    }
    isEnabled(prop) {
        const value = this.ad.configuration[prop];
        if (typeof value === 'boolean') {
            return value;
        }
        return true;
    }
}

class Responsive extends GenericPlugin {
    constructor() {
        super(...arguments);
        this.currentConfines = {};
        this.EVENT_KEY = 'resize';
        this.THROTTLE_DURATION = 250;
    }
    beforeCreate() {
        if (this.isRefreshDisabled()) {
            dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'Responsive Plugin', `Ad refresh is disabled.`);
            return;
        }
        this.determineCurrentBreakpoint();
        this.listener = debounce(() => {
            const { from = 0, to = 1 } = this.currentConfines;
            if (isBetween(window.innerWidth, from, to)) {
                return;
            }
            this.determineCurrentBreakpoint();
            dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'Responsive Plugin', `Viewport dimensions have entered a new breakpoint. Calling refresh.`);
            this.ad.refresh();
        }, this.THROTTLE_DURATION);
        window.addEventListener(this.EVENT_KEY, this.listener);
    }
    beforeClear() {
        dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'Responsive Plugin', `Removing window resize listener for ad.`);
        window.removeEventListener(this.EVENT_KEY, this.listener);
    }
    beforeDestroy() {
        dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'Responsive Plugin', `Removing window resize listener for ad.`);
        window.removeEventListener(this.EVENT_KEY, this.listener);
    }
    determineCurrentBreakpoint() {
        const { sizes = [], breakpoints } = this.ad.configuration;
        this.currentConfines = breakpointHandler(sizes, breakpoints);
    }
    isRefreshDisabled() {
        const { configuration } = this.ad;
        return (configuration.hasOwnProperty('refreshOnBreakpoint') && !configuration.refreshOnBreakpoint);
    }
}

return Responsive;

}());
//# sourceMappingURL=plugins.responsive.development.js.map
this._ADJS = this._ADJS || {};
this._ADJS.Plugins = this._ADJS.Plugins || {};
this._ADJS.Plugins.Sticky = (function () {
'use strict';

var LOG_LEVELS;
(function (LOG_LEVELS) {
    LOG_LEVELS["DEBUG"] = "log";
    LOG_LEVELS["INFO"] = "info";
    LOG_LEVELS["WARN"] = "warn";
    LOG_LEVELS["ERROR"] = "error";
})(LOG_LEVELS || (LOG_LEVELS = {}));

var dispatchEvent = (id, level, creator, message, eventData) => {
    const event = new CustomEvent(`adjs:${id}:event`, {
        detail: {
            level,
            creator,
            message,
            data: eventData,
        },
    });
    window.dispatchEvent(event);
};

/**
 * @param {function} cb Callback to be executed post timed throttle
 * @param {number} customDelay Time in Milliseconds
 */
let previousCall = 0;
var throttle = (cb, customDelay) => {
    const delay = customDelay || 200;
    const currentTime = new Date().getTime();
    if (currentTime - previousCall <= delay) {
        return;
    }
    previousCall = currentTime;
    return cb();
};

class GenericPlugin {
    constructor(ad) {
        if (!ad) {
            throw new Error(`An ad must be passed into the GenericPlugin class. If your Plugin inherits from GenericPlugin
		and overrides the constructor make sure you are calling "super" and that you are passing in an
		instance of an ad as the first parameter. Alternatively, you can hook into the onCreate method
		which gets called by the constructor.

		Example:
		js
		  class ExamplePlugin extends GenericPlugin {
			onCreate() {
			  console.log('Example Plugin Started Succesfully');
			}
		  }

		  // Or

		  class ExamplePlugin extends GenericPlugin {
			constructor(ad) {
			  super(ad);

			  console.log('Example Plugin Started Succesfully');
			}
		  }`);
        }
        this.ad = ad;
    }
    get name() {
        return this.constructor.name;
    }
    isEnabled(prop) {
        const value = this.ad.configuration[prop];
        if (typeof value === 'boolean') {
            return value;
        }
        return true;
    }
}

function getElementOffset(el) {
    const rect = el.getBoundingClientRect();
    const scrollLeft = window.pageXOffset || document.documentElement.scrollLeft;
    const scrollTop = window.pageYOffset || document.documentElement.scrollTop;
    return { top: rect.top + scrollTop, left: rect.left + scrollLeft, right: rect.right };
}
class Sticky extends GenericPlugin {
    onCreate() {
        if (!this.isEnabled('sticky')) {
            return;
        }
        const { container, configuration } = this.ad;
        const { stickyOffset = 0 } = configuration;
        this.originalStyle = container.style;
        container.style.position = 'sticky';
        container.style.top = `${stickyOffset}px`;
        if (container.parentElement) {
            this.boundary = getElementOffset(container.parentElement);
            if (navigator.appName === 'Netscape') {
                const ua = navigator.userAgent;
                const re = new RegExp('Trident/.*rv:([0-9]{1,}[\\.0-9]{0,})');
                if (re.exec(ua) != null) {
                    const rv = parseInt(RegExp.$1, 10);
                    if (rv === 11) {
                        this.handleIE(stickyOffset);
                    }
                }
            }
        }
        else {
            return;
        }
        dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'Sticky Plugin', `Sticky container added to ad.`);
    }
    cleanup() {
        const { container } = this.ad;
        container.style.position = this.originalStyle.position;
        container.style.top = this.originalStyle.top;
        container.style.transform = this.originalStyle.transform;
        if (this.listener) {
            window.removeEventListener('scroll', this.listener);
        }
    }
    onDestroy() {
        if (!this.isEnabled('sticky')) {
            return;
        }
        this.cleanup();
        dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'Sticky Plugin', `Removed sticky container from ad.`);
    }
    handleIE(offset) {
        const { container } = this.ad;
        const parentElement = container.parentElement;
        if (!parentElement) {
            throw new Error('Parent element required for sticky plugin.');
        }
        if (this.boundary && this.boundary.top < window.scrollY) {
            this.boundary = getElementOffset(parentElement);
            container.style.position = 'fixed';
            container.style.top = `${offset}px`;
            container.style.transform = 'translateX(-50%)';
        }
        this.listener = window.addEventListener('scroll', () => {
            throttle(() => {
                this.boundary = getElementOffset(parentElement);
                if (this.boundary.top > window.scrollY) {
                    container.style.position = this.originalStyle.position;
                    container.style.top = this.originalStyle.top;
                    container.style.transform = this.originalStyle.transform;
                }
                else if ((window.innerHeight + window.scrollY) >= document.body.offsetHeight) {
                    container.style.top = '';
                    container.style.bottom = '0px';
                    container.style.transform = 'translateX(-50%)';
                }
                else {
                    container.style.position = 'fixed';
                    container.style.top = `${offset}px`;
                    container.style.transform = 'translateX(-50%)';
                }
            });
        });
    }
}

return Sticky;

}());
//# sourceMappingURL=plugins.sticky.development.js.map
