this._ADJS = this._ADJS || {};
this._ADJS.Plugins = this._ADJS.Plugins || {};
this._ADJS.Plugins.Sticky = (function () {
'use strict';

var LOG_LEVELS;
(function (LOG_LEVELS) {
    LOG_LEVELS["DEBUG"] = "log";
    LOG_LEVELS["INFO"] = "info";
    LOG_LEVELS["WARN"] = "warn";
    LOG_LEVELS["ERROR"] = "error";
})(LOG_LEVELS || (LOG_LEVELS = {}));

var dispatchEvent = (id, level, creator, message, eventData) => {
    const event = new CustomEvent(`adjs:${id}:event`, {
        detail: {
            level,
            creator,
            message,
            data: eventData,
        },
    });
    window.dispatchEvent(event);
};

/**
 * @param {function} cb Callback to be executed post timed throttle
 * @param {number} customDelay Time in Milliseconds
 */
let previousCall = 0;
var throttle = (cb, customDelay) => {
    const delay = customDelay || 200;
    const currentTime = new Date().getTime();
    if (currentTime - previousCall <= delay) {
        return;
    }
    previousCall = currentTime;
    return cb();
};

class GenericPlugin {
    constructor(ad) {
        if (!ad) {
            throw new Error(`An ad must be passed into the GenericPlugin class. If your Plugin inherits from GenericPlugin
		and overrides the constructor make sure you are calling "super" and that you are passing in an
		instance of an ad as the first parameter. Alternatively, you can hook into the onCreate method
		which gets called by the constructor.

		Example:
		js
		  class ExamplePlugin extends GenericPlugin {
			onCreate() {
			  console.log('Example Plugin Started Succesfully');
			}
		  }

		  // Or

		  class ExamplePlugin extends GenericPlugin {
			constructor(ad) {
			  super(ad);

			  console.log('Example Plugin Started Succesfully');
			}
		  }`);
        }
        this.ad = ad;
    }
    get name() {
        return this.constructor.name;
    }
    isEnabled(prop) {
        const value = this.ad.configuration[prop];
        if (typeof value === 'boolean') {
            return value;
        }
        return true;
    }
}

function getElementOffset(el) {
    const rect = el.getBoundingClientRect();
    const scrollLeft = window.pageXOffset || document.documentElement.scrollLeft;
    const scrollTop = window.pageYOffset || document.documentElement.scrollTop;
    return { top: rect.top + scrollTop, left: rect.left + scrollLeft, right: rect.right };
}
class Sticky extends GenericPlugin {
    onCreate() {
        if (!this.isEnabled('sticky')) {
            return;
        }
        const { container, configuration } = this.ad;
        const { stickyOffset = 0 } = configuration;
        this.originalStyle = container.style;
        container.style.position = 'sticky';
        container.style.top = `${stickyOffset}px`;
        if (container.parentElement) {
            this.boundary = getElementOffset(container.parentElement);
            if (navigator.appName === 'Netscape') {
                const ua = navigator.userAgent;
                const re = new RegExp('Trident/.*rv:([0-9]{1,}[\\.0-9]{0,})');
                if (re.exec(ua) != null) {
                    const rv = parseInt(RegExp.$1, 10);
                    if (rv === 11) {
                        this.handleIE(stickyOffset);
                    }
                }
            }
        }
        else {
            return;
        }
        dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'Sticky Plugin', `Sticky container added to ad.`);
    }
    cleanup() {
        const { container } = this.ad;
        container.style.position = this.originalStyle.position;
        container.style.top = this.originalStyle.top;
        container.style.transform = this.originalStyle.transform;
        if (this.listener) {
            window.removeEventListener('scroll', this.listener);
        }
    }
    onDestroy() {
        if (!this.isEnabled('sticky')) {
            return;
        }
        this.cleanup();
        dispatchEvent(this.ad.id, LOG_LEVELS.INFO, 'Sticky Plugin', `Removed sticky container from ad.`);
    }
    handleIE(offset) {
        const { container } = this.ad;
        const parentElement = container.parentElement;
        if (!parentElement) {
            throw new Error('Parent element required for sticky plugin.');
        }
        if (this.boundary && this.boundary.top < window.scrollY) {
            this.boundary = getElementOffset(parentElement);
            container.style.position = 'fixed';
            container.style.top = `${offset}px`;
            container.style.transform = 'translateX(-50%)';
        }
        this.listener = window.addEventListener('scroll', () => {
            throttle(() => {
                this.boundary = getElementOffset(parentElement);
                if (this.boundary.top > window.scrollY) {
                    container.style.position = this.originalStyle.position;
                    container.style.top = this.originalStyle.top;
                    container.style.transform = this.originalStyle.transform;
                }
                else if ((window.innerHeight + window.scrollY) >= document.body.offsetHeight) {
                    container.style.top = '';
                    container.style.bottom = '0px';
                    container.style.transform = 'translateX(-50%)';
                }
                else {
                    container.style.position = 'fixed';
                    container.style.top = `${offset}px`;
                    container.style.transform = 'translateX(-50%)';
                }
            });
        });
    }
}

return Sticky;

}());
//# sourceMappingURL=plugins.sticky.development.js.map
